import { canonicalStringify } from "../../cache/index.js";
var SuspenseCache = (function () {
    function SuspenseCache() {
        this.queries = new Map();
    }
    SuspenseCache.prototype.add = function (query, variables, _a) {
        var promise = _a.promise, observable = _a.observable;
        var variablesKey = this.getVariablesKey(variables);
        var map = this.queries.get(query) || new Map();
        var entry = {
            observable: observable,
            fulfilled: false,
            promise: promise
                .catch(function () {
            })
                .finally(function () {
                entry.fulfilled = true;
            }),
        };
        map.set(variablesKey, entry);
        this.queries.set(query, map);
        return entry;
    };
    SuspenseCache.prototype.lookup = function (query, variables) {
        var _a;
        return (_a = this.queries
            .get(query)) === null || _a === void 0 ? void 0 : _a.get(this.getVariablesKey(variables));
    };
    SuspenseCache.prototype.remove = function (query, variables) {
        var map = this.queries.get(query);
        if (!map) {
            return;
        }
        var key = this.getVariablesKey(variables);
        var entry = map.get(key);
        if (entry && !entry.observable.hasObservers()) {
            map.delete(key);
        }
        if (map.size === 0) {
            this.queries.delete(query);
        }
    };
    SuspenseCache.prototype.getVariablesKey = function (variables) {
        return canonicalStringify(variables || Object.create(null));
    };
    return SuspenseCache;
}());
export { SuspenseCache };
//# sourceMappingURL=SuspenseCache.js.map