'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

require('../utilities/globals');
var context = require('./context');
var hooks = require('./hooks');
var parser = require('./parser');
var cache = require('../cache');

var SuspenseCache = (function () {
    function SuspenseCache() {
        this.queries = new Map();
    }
    SuspenseCache.prototype.add = function (query, variables, _a) {
        var promise = _a.promise, observable = _a.observable;
        var variablesKey = this.getVariablesKey(variables);
        var map = this.queries.get(query) || new Map();
        var entry = {
            observable: observable,
            fulfilled: false,
            promise: promise
                .catch(function () {
            })
                .finally(function () {
                entry.fulfilled = true;
            }),
        };
        map.set(variablesKey, entry);
        this.queries.set(query, map);
        return entry;
    };
    SuspenseCache.prototype.lookup = function (query, variables) {
        var _a;
        return (_a = this.queries
            .get(query)) === null || _a === void 0 ? void 0 : _a.get(this.getVariablesKey(variables));
    };
    SuspenseCache.prototype.remove = function (query, variables) {
        var map = this.queries.get(query);
        if (!map) {
            return;
        }
        var key = this.getVariablesKey(variables);
        var entry = map.get(key);
        if (entry && !entry.observable.hasObservers()) {
            map.delete(key);
        }
        if (map.size === 0) {
            this.queries.delete(query);
        }
    };
    SuspenseCache.prototype.getVariablesKey = function (variables) {
        return cache.canonicalStringify(variables || Object.create(null));
    };
    return SuspenseCache;
}());

exports.ApolloConsumer = context.ApolloConsumer;
exports.ApolloProvider = context.ApolloProvider;
exports.getApolloContext = context.getApolloContext;
exports.resetApolloContext = context.resetApolloContext;
exports.DocumentType = parser.DocumentType;
exports.operationName = parser.operationName;
exports.parser = parser.parser;
exports.SuspenseCache = SuspenseCache;
for (var k in hooks) {
    if (k !== 'default' && !exports.hasOwnProperty(k)) exports[k] = hooks[k];
}
//# sourceMappingURL=react.cjs.map
