'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

require('../utilities/globals');
var tslib = require('tslib');
var React = require('react');
var core$1 = require('../core');
var cache = require('../cache');
var context = require('../react/context');
var core = require('./core');

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        for (var k in e) {
            n[k] = e[k];
        }
    }
    n["default"] = e;
    return Object.freeze(n);
}

var React__namespace = /*#__PURE__*/_interopNamespace(React);

var SuspenseCache = (function () {
    function SuspenseCache() {
        this.queries = new Map();
    }
    SuspenseCache.prototype.add = function (query, variables, _a) {
        var promise = _a.promise, observable = _a.observable;
        var variablesKey = this.getVariablesKey(variables);
        var map = this.queries.get(query) || new Map();
        var entry = {
            observable: observable,
            fulfilled: false,
            promise: promise
                .catch(function () {
            })
                .finally(function () {
                entry.fulfilled = true;
            }),
        };
        map.set(variablesKey, entry);
        this.queries.set(query, map);
        return entry;
    };
    SuspenseCache.prototype.lookup = function (query, variables) {
        var _a;
        return (_a = this.queries
            .get(query)) === null || _a === void 0 ? void 0 : _a.get(this.getVariablesKey(variables));
    };
    SuspenseCache.prototype.remove = function (query, variables) {
        var map = this.queries.get(query);
        if (!map) {
            return;
        }
        var key = this.getVariablesKey(variables);
        var entry = map.get(key);
        if (entry && !entry.observable.hasObservers()) {
            map.delete(key);
        }
        if (map.size === 0) {
            this.queries.delete(query);
        }
    };
    SuspenseCache.prototype.getVariablesKey = function (variables) {
        return cache.canonicalStringify(variables || Object.create(null));
    };
    return SuspenseCache;
}());

var MockedProvider = (function (_super) {
    tslib.__extends(MockedProvider, _super);
    function MockedProvider(props) {
        var _this = _super.call(this, props) || this;
        var _a = _this.props, mocks = _a.mocks, addTypename = _a.addTypename, defaultOptions = _a.defaultOptions, cache$1 = _a.cache, resolvers = _a.resolvers, link = _a.link, suspenseCache = _a.suspenseCache;
        var client = new core$1.ApolloClient({
            cache: cache$1 || new cache.InMemoryCache({ addTypename: addTypename }),
            defaultOptions: defaultOptions,
            link: link || new core.MockLink(mocks || [], addTypename),
            resolvers: resolvers,
        });
        _this.state = {
            client: client,
            suspenseCache: suspenseCache || new SuspenseCache()
        };
        return _this;
    }
    MockedProvider.prototype.render = function () {
        var _a = this.props, children = _a.children, childProps = _a.childProps;
        var _b = this.state, client = _b.client, suspenseCache = _b.suspenseCache;
        return React__namespace.isValidElement(children) ? (React__namespace.createElement(context.ApolloProvider, { client: client, suspenseCache: suspenseCache }, React__namespace.cloneElement(React__namespace.Children.only(children), tslib.__assign({}, childProps)))) : null;
    };
    MockedProvider.prototype.componentWillUnmount = function () {
        this.state.client.stop();
    };
    MockedProvider.defaultProps = {
        addTypename: true
    };
    return MockedProvider;
}(React__namespace.Component));

exports.MockedProvider = MockedProvider;
for (var k in core) {
    if (k !== 'default' && !exports.hasOwnProperty(k)) exports[k] = core[k];
}
//# sourceMappingURL=testing.cjs.map
